﻿/*  (c) 2008 Benjamin Schroeter
 *  You are allowed to use this code in all your commercial, non-commercial, open and closed source projects.
 *  But you have to leave this Copyright message in all files. This code is provided "as is" without warranty of any kind.
 *  If you have any enhancements or bug reports I will appreciate a short feedback.
 *  For more information visit the article to this code on my blog at 
 *  http://www.ticklishtechs.net/2008/02/29/connectors-between-compartment-shape-entries-with-dsl-tools-part-1/ */
using System.Collections.Generic;
using Microsoft.VisualStudio.Modeling.Diagrams;

namespace BenjaminSchroeter.Dsl.CompartmentMapping.Extensions
{
    /// <summary>
    /// Some extention methods
    /// </summary>
    internal static class ExtensionMethods
    {
        /// <summary>
        /// Findes the first represented element of a DiagramItem
        /// </summary>
        /// <param name="item">a DiagramItem</param>
        /// <returns>the first represented element or null if there is no represented element</returns>
        public static object FirstRepresentedElement(this DiagramItem item)
        {
            foreach (object o in item.RepresentedElements)
                return o;

            return null;
        }

        /// <summary>
        /// Gets all children of a ShapeElement as a List
        /// </summary>
        /// <remarks>This is a wrapper for easy use of the FindFirstChild() and FindNextChild() methodes</remarks>
        /// <param name="shape">a ShapeElement</param>
        /// <returns>the children of the given ShapeElement</returns>
        public static IList<DiagramItem> Children(this ShapeElement shape)
        {
            List<DiagramItem> list = new List<DiagramItem>();
            DiagramItem x = shape.FindFirstChild(false);
            while(x != null)
            {
                list.Add(x);
                x = shape.FindNextChild(x, false);
            }

            return list;
        }
    }
}