﻿using System;
using BenjaminSchroeter.Dsl.CompartmentMapping;
using Microsoft.VisualStudio.Modeling;

namespace BenjaminSchroeter.CompartmentMappingExample
{
    public partial class Parent1ReferencesParent2BuilderInstance
    {
        /// <summary>
        /// Checks if the given connection has the entry as source
        /// </summary>
        /// <remarks>You have to implement this method, because only you can know how to store this information in your connection class.</remarks>
        /// <param name="entry">the entry if a compartment shape</param>
        /// <param name="connection">the connection</param>
        /// <returns>true, if the connection has the entry as source</returns>
        public override bool IsEntryConnectionSource(Entry1 entry, Parent1ReferencesParent2 connection)
        {
            if (entry == null)
                return false;
            return connection.fromEntry.Equals(entry.Guid);
        }

        /// <summary>
        /// Checks if the given connection has the entry as target
        /// </summary>
        /// <remarks>You have to implement this method, because only you can know how to store this information in your connection class.</remarks>
        /// <param name="entry">the entry if a compartment shape</param>
        /// <param name="connection">the connection</param>
        /// <returns>true, if the connection has the entry as target</returns>
        public override bool IsEntryConnectionTarget(Entry2 entry, Parent1ReferencesParent2 connection)
        {
            if (entry == null)
                return false;
            return connection.toEntry.Equals(entry.Guid);
        }

        /// <summary>
        /// Make a connection between the given pair of source and target elements
        /// </summary>
        /// <returns>A link representing the created connection</returns>        
        /// <remarks>This method is only called if both, source and targent, are compartment shapes.</remarks>
        /// <param name="source">The model element to use as the source of the connection</param>
        /// <param name="sourcePartType">The type of the source. (head or entry)</param>
        /// <param name="sourceEntry">The model element to use as the source entry of the conneciton</param>
        /// <param name="target">The model element to use as the target of the connection</param>
        /// <param name="targetPartType">The type of the target. (head or entry)</param>
        /// <param name="targetEntry">The model elment to use as the target entry of the connection</param>
        /// <returns>A link representing the created connection</returns>
        protected override ElementLink CreateElementLink(Parent1 source,
                                                            SelectedCompartmentPartType sourcePartType,
                                                            Entry1 sourceEntry,
                                                            Parent2 target,
                                                            SelectedCompartmentPartType targetPartType,
                                                            Entry2 targetEntry)
        {
            Parent1ReferencesParent2 result = new Parent1ReferencesParent2(source, target);
            result.fromEntry = sourceEntry.Guid;
            result.toEntry = targetEntry.Guid;
            return result;
        }
    }
}