/* You have to implement the following partial classes and methods in another .cs file:
---------------------------------------------------------------------------------------
using System;
using BenjaminSchroeter.Dsl.CompartmentMapping;
using Microsoft.VisualStudio.Modeling;

namespace BenjaminSchroeter.CompartmentMappingExample
{
    public partial class Parent1ReferencesParent2BuilderInstance
    {
        /// <summary>
        /// Checks if the given connection has the entry as source
        /// </summary>
        /// <remarks>You have to implement this method, because only you can know how to store this information in your connection class.</remarks>
        /// <param name="entry">the entry if a compartment shape</param>
        /// <param name="connection">the connection</param>
        /// <returns>true, if the connection has the entry as source</returns>
        public override bool IsEntryConnectionSource(Entry1 entry, Parent1ReferencesParent2 connection)
        {
        
        }
        
        /// <summary>
        /// Checks if the given connection has the entry as target
        /// </summary>
        /// <remarks>You have to implement this method, because only you can know how to store this information in your connection class.</remarks>
        /// <param name="entry">the entry if a compartment shape</param>
        /// <param name="connection">the connection</param>
        /// <returns>true, if the connection has the entry as target</returns>
        public override bool IsEntryConnectionTarget(Entry2 entry, Parent1ReferencesParent2 connection)
        {
        
        }
        
        /// <summary>
        /// Make a connection between the given pair of source and target elements
        /// </summary>
        /// <returns>A link representing the created connection</returns>        
        /// <remarks>This method is only called if both, source and targent, are compartment shapes.</remarks>
        /// <param name="source">The model element to use as the source of the connection</param>
		/// <param name="sourcePartType">The type of the source. (head or entry)</param>
        /// <param name="sourceEntry">The model element to use as the source entry of the conneciton</param>
        /// <param name="target">The model element to use as the target of the connection</param>
        /// <param name="targetPartType">The type of the target. (head or entry)</param>
        /// <param name="targetEntry">The model elment to use as the target entry of the connection</param>
        /// <returns>A link representing the created connection</returns>
        protected override ElementLink CreateElementLink(Parent1 source,
 																	SelectedCompartmentPartType sourcePartType,
																	Entry1 sourceEntry, 
																	Parent2 target, 
																	SelectedCompartmentPartType targetPartType,
																	Entry2 targetEntry)
        {
            Parent1ReferencesParent2 result = new Parent1ReferencesParent2(source, target);
			// TODO: store somewhere in the Connection the source and target element
            return result;
        }
	}
}
---------------------------------------------------------------------------------------
*/
using System;
using System.Diagnostics;
using BenjaminSchroeter.Dsl.CompartmentMapping;
using Microsoft.VisualStudio.Modeling;
using Microsoft.VisualStudio.Modeling.Diagrams.GraphObject;

namespace BenjaminSchroeter.CompartmentMappingExample
{
	#region Code for Parent1ReferencesParent2
    partial class Connector
    {
        public override void OnInitialize()
        {
			this.FixedFrom = VGFixedCode.Caller;
			this.FixedTo = VGFixedCode.Caller;
			base.OnInitialize();
        }

        public override bool CanManuallyRoute
        {
            get
            {
                return false;
            }
        }
    }

    [RuleOn(typeof(Connector), FireTime = TimeToFire.LocalCommit)]
    public class ConnectorAddRule : CompartmentMappingAddRuleBase
    {
    }

    [RuleOn(typeof(Entry1), FireTime = TimeToFire.LocalCommit)]
    [RuleOn(typeof(Entry2), FireTime = TimeToFire.LocalCommit)]
    public class SourceOrTargetOfParent1ReferencesParent2DeletingRule : CompartmentEntryDeletingRuleBase<Parent1, Entry1, Parent1ReferencesParent2, Parent2, Entry2>
    {
        protected override CompartmentMappingBuilderBase<Parent1, Entry1, Parent1ReferencesParent2, Parent2, Entry2> GetBuilderInstance()
        {
            return Parent1ReferencesParent2Builder.builder;
        }
    }   
    
    public class ConnectorRouter : CompartmentMappingRouterBase<Parent1, Entry1, Parent1ReferencesParent2, Parent2, Entry2>
    {
        protected override CompartmentMappingBuilderBase<Parent1, Entry1, Parent1ReferencesParent2, Parent2, Entry2> GetBuilderInstance()
        {
            return Parent1ReferencesParent2Builder.builder;
        }
    }
    
    public partial class Parent1ReferencesParent2Builder
    {
        internal static readonly Parent1ReferencesParent2BuilderInstance builder = new Parent1ReferencesParent2BuilderInstance();

        public static bool CanAcceptSource(ModelElement candidate)
        {
            return builder.CanAcceptSource(candidate);
        }

        public static bool CanAcceptTarget(ModelElement candidate)
        {
            return builder.CanAcceptTarget(candidate);
        }

        public static bool CanAcceptSourceAndTarget(ModelElement candidateSource, ModelElement candidateTarget)
        {
            return builder.CanAcceptSourceAndTarget(candidateSource, candidateTarget);
        }

        public static ElementLink Connect(ModelElement source, ModelElement target)
        {
            return builder.Connect(source, target);
        }
    }
    
    public partial class Parent1ReferencesParent2BuilderInstance : CompartmentMappingBuilderBase<Parent1, Entry1, Parent1ReferencesParent2, Parent2, Entry2>
	{	
		internal Parent1ReferencesParent2BuilderInstance() : base(false,		/* def.Source.IsRegularShape */
																false,		/* def.Target.IsRegularShape */
																false,		/* def.Source.AllowHead */
																false,		/* def.Target.AllowHead */
																false		/* def.Connection.AllowSelfReference */		)
		{
		}
		
		// to prevent the user from overriding the following methods
		protected override bool CanAcceptAsCompartmentSource(Parent1 candidate)
		{
		    Debug.Fail(ERROR_MESSAGE_WRONG_METHOD_CALLED_CODEGEN_SHOULD_PREVENT);
            throw new InvalidOperationException(ERROR_MESSAGE_WRONG_METHOD_CALLED_CODEGEN_SHOULD_PREVENT);
		}
		
        protected override ElementLink CreateElementLink(Parent1 source, Parent2 target, SelectedCompartmentPartType targetPartType, Entry2 targetEntry)
        {
			return base.CreateElementLink(source,target,targetPartType,targetEntry);
        }
        
        protected override bool CanAcceptAsCompartmentSourceAndTarget(Parent1 sourceElement, Parent2 targetElement, SelectedCompartmentPartType targetPartType, Entry2 targetEntry)
		{
		    Debug.Fail(ERROR_MESSAGE_WRONG_METHOD_CALLED_CODEGEN_SHOULD_PREVENT);
            throw new InvalidOperationException(ERROR_MESSAGE_WRONG_METHOD_CALLED_CODEGEN_SHOULD_PREVENT);
		}
		
        protected override ElementLink CreateElementLink(Parent1 source, SelectedCompartmentPartType sourcePartType, Entry1 sourceEntry, Parent2 target)
        {
			return base.CreateElementLink(source,sourcePartType,sourceEntry,target);
        }
        
        protected override bool CanAcceptAsCompartmentSourceAndTarget(Parent1 sourceElement, SelectedCompartmentPartType sourcePartType, Entry1 sourceEntry, Parent2 targetElement)
		{
		    Debug.Fail(ERROR_MESSAGE_WRONG_METHOD_CALLED_CODEGEN_SHOULD_PREVENT);
            throw new InvalidOperationException(ERROR_MESSAGE_WRONG_METHOD_CALLED_CODEGEN_SHOULD_PREVENT);
		}
	}	
	#endregion

	#region Code for CompartmentShape1
    partial class CompartmentShape1 : ICompartmentMouseActionTrackable
    {
        private readonly CompartmentMouseTrack mouseTrack = new CompartmentMouseTrack();

        public CompartmentMouseTrack MouseTrack
        {
            get { return mouseTrack; }
        }

        public override void OnShapeInserted()
        {
            mouseTrack.ShapeInserted(this);
            base.OnShapeInserted();
        }
        
        public override void OnAbsoluteBoundsChanged(Microsoft.VisualStudio.Modeling.Diagrams.AbsoluteBoundsChangedEventArgs e)
        {
            CompartmentMappingUtil.RerouteCompartmentMappings(this);
            base.OnAbsoluteBoundsChanged(e);
        }
    }
	#endregion
	#region Code for CompartmentShape2
    partial class CompartmentShape2 : ICompartmentMouseActionTrackable
    {
        private readonly CompartmentMouseTrack mouseTrack = new CompartmentMouseTrack();

        public CompartmentMouseTrack MouseTrack
        {
            get { return mouseTrack; }
        }

        public override void OnShapeInserted()
        {
            mouseTrack.ShapeInserted(this);
            base.OnShapeInserted();
        }
        
        public override void OnAbsoluteBoundsChanged(Microsoft.VisualStudio.Modeling.Diagrams.AbsoluteBoundsChangedEventArgs e)
        {
            CompartmentMappingUtil.RerouteCompartmentMappings(this);
            base.OnAbsoluteBoundsChanged(e);
        }
    }
	#endregion
}
